<?php
namespace Teknisa\CADEST\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\CADEST\Services;

class Layout extends CrudWithParameterBag {
    
    protected $util,
              $parameterBag,
              $layoutService, 
              $fileUploaderService; 
    
    protected $dataSourceNameArray = array (
        'filterGrupoProduto'         => 'vnd80600_configterminalcaixa_layoutProduto',
        'filterProdutos'             => 'vnd80600_configterminalcaixa_produtos_registraveis',
        'filterProdutos_getPosicoes' => 'vnd80600_configterminalcaixa_layoutGrupo_getPosicoes',
        'filterGrupo'                => 'vnd80600_configterminalcaixa_layoutGrupo',
        'filterLayout'               => 'findConfigTela'
    );
    
    protected $parameterBagColumnsArray = array (
        'filterLayout'               =>  array('NRORG', 'CDFILIAL', 'CDLOJA'),
        'filterGrupo'                =>  array('NRORG', 'CDFILIAL', 'CDPRODUTO'),
        'filterLoja_Tela'            =>  array('NRORG', 'CDFILIAL', 'CDLOJA', 'NRCONFTELA'),
        'filterProdutos'             =>  array('NRORG', 'CDFILIAL'),
        'filterProdutosCadAud'       =>  array('NRORG', 'CDFILIAL'),
        'filterProductOnLayout'      =>  array('NRORG', 'CDPRODUTO')
    );

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, Services\Layout $layoutService, Services\FileUploaderService $fileUploaderService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->layoutService = $layoutService;
        $this->fileUploaderService = $fileUploaderService;
    }
    
    public function findGrupoProduto (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupoProduto'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterProductOnLayout'];
        parent::find($request, $response);
    }
    
    public function saveGrupoProduto(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        if(count($rows) === 1 ) {
            $row = $rows[0];
            $this->fileUploaderService->setImageForSave($row, 'DSENDEIMG', 'IMAGE');
            
            $this->dataSourceName = $this->dataSourceNameArray['filterGrupoProduto'];
            if(  array_key_exists('ANTG_NRBUTTON', $row) &&
                 $row['ANTG_NRBUTTON'] !== '' && 
                ($row['NRBUTTON']      !== $row['ANTG_NRBUTTON'])) {
                $aux = $row['NRBUTTON'];     
                $row['NRBUTTON'] = $row['ANTG_NRBUTTON'];
                self::delete($request, $response);
                $row['NRBUTTON'] = $aux;
                $row['__is_new'] = true;
            }
            self::saveDefaulLayout($row, $response, $this->dataSourceNameArray['filterGrupoProduto']);
        } else {
            throw new \Exception('Só é possível salvar um registro de cada vez.');
        }
    }
    
    public function saveDefaulLayout ($row, $response, $dataSourceNameSave) {
        try {
            $dataSet = new DataSet($dataSourceNameSave, array($row));
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($dataSourceNameSave, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }    
    }
    
    public function findPosicaoDisponivel (DTO\Request\Filter $request, DTO\Response $response) {
        
        
        $this->dataSourceName = $this->dataSourceNameArray['filterProdutos_getPosicoes'];
        $retorno = [];
        $conditions = $request->getFilterCriteria()->getConditions();
        
        if(count($conditions) > 0) {
            $params = array(
    			':NRORG'  		  => $conditions[0]['value'],
    			':CDFILIAL'       => $conditions[1]['value'],
    			':NRCONFTELA'     => $conditions[2]['value'],
    			':IDTPBUTONAUX'   => $conditions[3]['value'],
    			':NRBUTTONAUX'    => $conditions[4]['value'],
    			':NRPGCONFTAUX'   => $conditions[5]['value'],
    			':IDTPBUTONAUX2'  => $conditions[6]['value'],
    			':NRBUTTONAUX2'   => $conditions[7]['value'],
    			':NRPGCONFTELA'   => $conditions[8]['value'],
    			':CDGRUPO'        => $conditions[9]['value'],
    			':PROPRIAPOSICAO' => $conditions[10]['value']
    		);
            
            if( empty($params[':IDTPBUTONAUX2']) )
                $params[':IDTPBUTONAUX2'] = "0";
            
            if( empty($params[':NRBUTTONAUX2']) )
                $params[':NRBUTTONAUX2'] = "0";
                
            if( empty($params[':CDGRUPO']) )
                $params[':CDGRUPO'] = "0";
                
            $resultado = $this->layoutService->findPosicaoDisponivelBD($params);
            
            $posicoes = array("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13",
                              "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26",
                              "27", "28", "29", "30", "31", "32", "33", "34", "35", "36");
            
            foreach ($resultado as $posicao) {
                for($i = 0;  $i <= sizeof($posicoes) - 1 ; $i++) {
                    if($posicao['NRBUTTON'] == $posicoes[$i]) {
                        array_splice($posicoes, $i, 1);
                    }
                }
            }
            
            foreach($posicoes as $pos) {
                array_push($retorno, ["NRBUTTON" => $pos]);
            }
        }
        
        $response->addDataSet(new DataSet($this->dataSourceName, $retorno));
    }
    
    public function findProdutos(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterProdutos'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterProdutos'];
        parent::find($request, $response);
    }
    
    public function findLayout (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterLayout'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterLayout'];
        parent::find($request, $response);
    }
    
    public function findGrupo (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupo'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterLoja_Tela'];
        parent::find($request, $response);
    }
    
    public function deleteGrupoProduto(DTO\Request\DataSet $request, DTO\Response $response) {
         try {
            $dataSet = $request->getDataSet();
            foreach($dataSet->getRows() as &$row){
                $row['__is_new'] = false;
            }
            $this->dataSourceName = $this->dataSourceNameArray['filterGrupoProduto'];
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
            $nrDeletedRows = count($deletedRows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    

    
}